"""pytmux - A simple wrapper around tmux.

Usage:
  pytmux list
  pytmux run <config>
  pytmux edit <config> [--copy <other_config>]
  pytmux doctor
  pytmux -h | --help
  pytmux --version

Commands:
  list            Lists all configs.
  run <config>    Runs the specified config.
  edit <config>   Edits the specified config, with --copy will make a new
                  config based on the one specified.
  doctor          Validates all of you configs.

Options:
  -h --help       Show this screen.
  --version       Show version.

"""
from os import makedirs, path

from docopt import docopt

from __init__ import __version__
from core import (config_dir, doctor_command, edit_config, list_configs,
                  run_config)


def main():
    arguments = docopt(__doc__, version='pytmux {}'.format(__version__))

    if not path.exists(config_dir):
        makedirs(config_dir)

    if arguments['list']:
        list_configs()
    elif arguments['run']:
        run_config(arguments['<config>'])
    elif arguments['edit']:
        edit_config(arguments['<config>'], arguments['--copy'],
                    arguments['<other_config>'])
    elif arguments['doctor']:
        doctor_command()



import json
from jsonschema import Draft3Validator
from utils import get_config_path


schema = {
    'type': 'object',
    'required': True,
    'additionalProperties': False,
    'properties': {
        'name': {
            'type': 'string',
            'required': True
        },
        'windows': {
            'type': 'array',
            'required': True,
            'minItems': 1,
            'items': {
                'type': 'object',
                'required': True,
                'additionalProperties': False,
                'properties': {
                    'name': {
                        'type': 'string'
                    },
                    'command': {
                        'type': 'string'
                    }
                }
            }
        }
    }
}


def validate_config(config):
    filename = get_config_path(config)
    try:
        to_validate = json.load(open(filename))
    except ValueError, e:
        print '{}: {}'.format(filename, e)
        return

    validator = Draft3Validator(schema)

    for error in sorted(validator.iter_errors(to_validate), key=str):
        print '{}: {}'.format(filename, error)



import os
import subprocess
from collections import OrderedDict


config_dir = os.path.expanduser('~/.pytmux/')


base_config = OrderedDict([
    ('name', 'example'),
    ('windows', [
        OrderedDict([
            ('name', 'editor'),
            ('command', 'emacs'),
        ]), {
            'name': 'default shell',
        }, {
            'command': 'redis-cli MONITOR',
        }, {}
    ])
])


def get_config_path(filename):
    return os.path.join(config_dir, '{}.json'.format(filename))


class Tmux(object):
    def __init__(self, session=None, socket=None):
        self.session = session

    def call(self, *args, **kwargs):
        command = ('tmux',)
        print command+args
        return subprocess.call(command+args, **kwargs)

    def create(self):
        # Work around bug in tmux where it wont let you start another session
        # from inside of a session.
        env = os.environ.get('TMUX')
        if env:
            del os.environ['TMUX']

        self.call('new-session', '-s', self.session, '-d')

        if env:
            os.environ['TMUX'] = env

    def send_keys(self, window, command):
        return self.call('send-keys',
                         '-t', '{}:{}.'.format(self.session, window),
                         command, '^M')

    def attach(self):
        if os.environ.get('TMUX'):
            self.call('switch', '-t', self.session)
        else:
            self.call('attach-session', '-t', self.session)

    def new_window(self, number, name):
        name_arg = ()
        if name:
            name_arg = ('-n', name)

        self.call('new-window', '-d', '-k',
                  '-t', '{}:{}'.format(self.session, number),
                  *name_arg)



import json
import os
import shutil
import subprocess

import envoy

from schema import validate_config
from utils import base_config, config_dir, get_config_path, Tmux


def list_configs():
    configs = [os.path.splitext(fn)[0] for fn in os.listdir(config_dir)
               if os.path.splitext(fn)[1] == '.json']
    print 'Configs:'
    print '\n'.join(configs)


def run_config(config):
    try:
        settings = json.load(open(get_config_path(config)))
    except IOError, e:
        print '{}: "{}"'.format(e.strerror, e.filename)
        return
    except ValueError, e:
        print 'JSON in "{}" is not valid'.format(get_config_path(config))
        return

    tmux = Tmux(settings['name'], settings.get('socket'))

    make_session = tmux.call('has-session', '-t', settings['name'],
                             stderr=subprocess.PIPE)

    base_index = envoy.run('tmux show-options -g base-index').std_out or 0
    if base_index:
        base_index = int(base_index.strip()[-1])

    if make_session:
        tmux.create()

        # Create windows
        for index, window in enumerate(settings['windows'], base_index):
            tmux.new_window(index, window.get('name'))
            if 'command' in window:
                tmux.send_keys(index, window['command'])

    tmux.attach()



def edit_config(config, copy, other_config):
    editor = os.environ.get('EDITOR', 'vi')
    filename = get_config_path(config)
    if copy and other_config:
        file_to_copy = get_config_path(other_config)
        shutil.copyfile(file_to_copy, filename)
    elif not os.path.exists(filename):
        with open(filename, 'w') as f:
            f.write(json.dumps(base_config, indent=2))
    subprocess.call('{0} {1}'.format(editor, filename), shell=True)


def doctor_command():
    configs = [os.path.splitext(fn)[0] for fn in os.listdir(config_dir)
               if os.path.splitext(fn)[1] == '.json']
    for conf in configs:
        validate_config(conf)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import pytmux

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

required = [
    'docopt==0.6.1',
    'envoy==0.0.2',
    'jsonschema==1.1.0',
]

setup(
    name='pytmux',
    version=pytmux.__version__,
    description='A simple wrapper for tmux.',
    long_description=(open('README.rst').read() + '\n\n' +
                      open('HISTORY.rst').read()),
    author='Wraithan (Chris McDonald)',
    author_email='xwraithanx@gmail.com',
    url='https://github.com/wraithan/pytmux',
    packages=['pytmux'],
    install_requires=required,
    license='Apache License, Version 2.0',
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
    entry_points={
        'console_scripts': [
            'pytmux = pytmux.cli:main',
        ],
    },
)
